<?php
/***********************************************************************

  Copyright (C) 2002-2005  Rickard Andersson (rickard@punbb.org)

  This file is part of PunBB.

  PunBB is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published
  by the Free Software Foundation; either version 2 of the License,
  or (at your option) any later version.

  PunBB is distributed in the hope that it will be useful, but
  WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston,
  MA  02111-1307  USA

************************************************************************/


// Tell header.php to use the admin template
define('PUN_ADMIN_CONSOLE', 1);

define('PUN_ROOT', './');
require PUN_ROOT.'include/common.php';
require PUN_ROOT.'include/common_admin.php';


if ($pun_user['g_id'] > PUN_MOD)
	message($lang_common['No permission']);


$action = isset($_GET['action']) ? $_GET['action'] : null;

// Check for upgrade
if ($action == 'check_upgrade')
{
	if (!ini_get('allow_url_fopen'))
		message('Impossible de vérifier les mises à jour tant que \'allow_url_fopen\' est désactivé sur ce système.');

	$fp = @fopen('http://www.punbb.org/latest_version', 'r');
	$latest_version = trim(@fread($fp, 16));
	@fclose($fp);

	if ($latest_version == '')
		message('La vérification de mise à jour a échouée pour une raison inconnue.');

	$cur_version = str_replace(array('.', 'dev', 'beta', ' '), '', strtolower($pun_config['o_cur_version']));
	$cur_version = (strlen($cur_version) == 2) ? intval($cur_version) * 10 : intval($cur_version);

	$latest_version = str_replace('.', '', strtolower($latest_version));
	$latest_version = (strlen($latest_version) == 2) ? intval($latest_version) * 10 : intval($latest_version);

	if ($cur_version >= $latest_version)
		message('Vous utilisez la dernière version de PunBB.');
	else
		message('Une nouvelle version de PunBB est sortie ! Vous pouvez télécharger cette dernière version sur <a href="http://www.punbb.org/">PunBB.org</a>.');
}

// Check for french upgrade
if ($action == 'check_upgrade_fr')
{
	if (!ini_get('allow_url_fopen'))
		message('Impossible de vérifier les mises à jour tant que \'allow_url_fopen\' est désactivé sur ce système.');

	$fp = @fopen('http://www.punbb.fr/latest_version', 'r');
	$latest_version = trim(@fread($fp, 16));
	@fclose($fp);

	if ($latest_version == '')
		message('La vérification de mise à jour a échouée pour une raison inconnue.');

	$cur_version = str_replace(array('.', 'dev', 'beta', ' '), '', strtolower($pun_config['o_cur_version_fr']));
	$cur_version = (strlen($cur_version) == 2) ? intval($cur_version) * 10 : intval($cur_version);

	$latest_version = str_replace('.', '', strtolower($latest_version));
	$latest_version = (strlen($latest_version) == 2) ? intval($latest_version) * 10 : intval($latest_version);

	if ($cur_version >= $latest_version)
		message('Vous utilisez la dernière version de PunBB en français.');
	else
		message('Une nouvelle version de PunBB en français est sortie ! Vous pouvez télécharger cette dernière version sur <a href="http://www.punbb.fr/">PunBB.fr</a>.');
}


// Show phpinfo() output
else if ($action == 'phpinfo' && $pun_user['g_id'] == PUN_ADMIN)
{
	// Is phpinfo() a disabled function?
	if (strpos(strtolower((string)@ini_get('disable_functions')), 'phpinfo') !== false)
		message('La fonction phpinfo() de PHP est désactivée sur ce serveur.');

	phpinfo();
	exit;
}


// Get the server load averages (if possible)
if (@file_exists('/proc/loadavg') && is_readable('/proc/loadavg'))
{
	// We use @ just in case
	$fh = @fopen('/proc/loadavg', 'r');
	$load_averages = @fread($fh, 64);
	@fclose($fh);

	$load_averages = @explode(' ', $load_averages);
	$server_load = isset($load_averages[2]) ? $load_averages[0].' '.$load_averages[1].' '.$load_averages[2] : 'Indisponible';
}
else if (!in_array(PHP_OS, array('WINNT', 'WIN32')) && preg_match('/averages?: ([0-9\.]+),[\s]+([0-9\.]+),[\s]+([0-9\.]+)/i', @exec('uptime'), $load_averages))
	$server_load = $load_averages[1].' '.$load_averages[2].' '.$load_averages[3];
else
	$server_load = 'Indisponible';


// Get number of current visitors
$result = $db->query('SELECT COUNT(user_id) FROM '.$db->prefix.'online WHERE idle=0') or error('Impossible de retrouver le total d\'utilisateurs en ligne', __FILE__, __LINE__, $db->error());
$num_online = $db->result($result);


// Get the database system version
switch ($db_type)
{
	case 'sqlite':
		$db_version = 'SQLite '.sqlite_libversion();
		break;

	default:
		$result = $db->query('SELECT VERSION()') or error('Impossible de retrouver les informations utilisateurs', __FILE__, __LINE__, $db->error());
		$db_version = $db->result($result);
		break;
}


// Collect some additional info about MySQL
if ($db_type == 'mysql' || $db_type == 'mysqli')
{
	$db_version = 'MySQL '.$db_version;

	// Calculate total db size/row count
	$result = $db->query('SHOW TABLE STATUS FROM `'.$db_name.'`') or error('Impossible de retrouver le statut des tables', __FILE__, __LINE__, $db->error());

	$total_records = $total_size = 0;
	while ($status = $db->fetch_assoc($result))
	{
		$total_records += $status['Rows'];
		$total_size += $status['Data_length'] + $status['Index_length'];
	}

	$total_size = $total_size / 1024;

	if ($total_size > 1024)
		$total_size = round($total_size / 1024, 2).' MB';
	else
		$total_size = round($total_size, 2).' KB';
}


// See if MMCache or PHPA is loaded
if (function_exists('mmcache'))
	$php_accelerator = '<a href="http://turck-mmcache.sourceforge.net/">Turck MMCache</a>';
else if (isset($_PHPA))
	$php_accelerator = '<a href="http://www.php-accelerator.co.uk/">ionCube PHP Accelerator</a>';
else
	$php_accelerator = 'N/A';


$page_title = pun_htmlspecialchars($pun_config['o_board_title']).' / Admin';
require PUN_ROOT.'header.php';

generate_admin_menu('index');

?>
	<div class="block">
		<h2>Administration Forum</h2>
		<div id="adintro" class="box">
			<div class="inbox">
				<p>Bienvenue sur le panneau de contrôles de  PunBB. Depuis cet espace vous pouvez contrôler les points essentiels de votre forum. Selon que vous êtes un administrateur ou un modérateur vous pouvez&#160;:<br /><br />
					&#160;- organiser les catégories et les forums.<br />
					&#160;- régler les principales options et préférences.<br />
					&#160;- contrôler les permissions pour les utilisateurs et les visiteurs.<br />
					&#160;- voir les statistiques des IP pour les utilisateurs.<br />
					&#160;- bannir des utilisateurs.<br />
					&#160;- censurer des mots.<br />
					&#160;- régler les rangs des utilisateurs.<br />
					&#160;- élaguer les anciens messages.<br />
					&#160;- traiter les messages signalés.
				</p>
			</div>
		</div>

		<h2 class="block2"><span>Statistiques</span></h2>
		<div id="adstats" class="box">
			<div class="inbox">
				<dl>
					<dt>PunBB version</dt>
					<dd>
						PunBB version française <?php echo $pun_config['o_cur_version_fr'] ?> basée sur PunBB <?php echo $pun_config['o_cur_version'] ?><br />
						<a href="admin_index.php?action=check_upgrade">Vérifier la version officielle</a> - <a href="admin_index.php?action=check_upgrade_fr">Vérifier la version française</a> <br />
						&copy; Copyright 2002, 2003, 2004, 2005 Rickard Andersson
					</dd>
					<dt>Exécution serveur</dt>
					<dd>
						<?php echo $server_load ?> (<?php echo $num_online ?> utilisateurs en ligne)
					</dd>
<?php if ($pun_user['g_id'] == PUN_ADMIN): ?>					<dt>Environnement</dt>
					<dd>
						Système d'exploitation&#160;: <?php echo PHP_OS ?><br />
						PHP&#160;: <?php echo phpversion() ?> - <a href="admin_index.php?action=phpinfo">Afficher infos</a><br />
						Accélérateur PHP&#160;: <?php echo $php_accelerator."\n" ?>
					</dd>
					<dt>Base de données</dt>
					<dd>
						<?php echo $db_version."\n" ?>
<?php if (isset($total_records) && isset($total_size)): ?>						<br />Lignes&#160;: <?php echo $total_records."\n" ?>
						<br />Taille&#160;: <?php echo $total_size."\n" ?>
<?php endif; endif; ?>					</dd>
				</dl>
			</div>
		</div>
	</div>
	<div class="clearer"></div>
</div>
<?php

require PUN_ROOT.'footer.php';
